@@ -1,5 +1,5 @@
 # - perl *-
-# $Id: cssh.pl 214 2007-11-28 22:11:03Z duncan_ferguson $
+# $Id: cssh.pl 220 2008-01-23 22:18:33Z duncan_ferguson $
 #
 # Script:
 #   $RCSfile$
@@ -50,7 +50,7 @@
 #     http://www.opensource.org/licenses/gpl-license.php
 #
 ############################################################################
-my $VERSION = '$Revision: 214 $ ($Date: 2007-11-28 22:11:03 +0000 (Wed, 28 Nov 2007) $)';
+my $VERSION = '$Revision: 220 $ ($Date: 2008-01-23 22:18:33 +0000 (Wed, 23 Jan 2008) $)';
 
 # Now tidy it up, but in such as way cvs doesn't kill the tidy up stuff
 $VERSION =~ s/\$Revision: //;
@@ -97,6 +97,8 @@ my %servers;     # hash of server cx info
 my $helper_script = "";
 my $xdisplay;
 my %keyboardmap;
+my $sysconfigdir = "/etc";
+my %ssh_hostnames;
 
 # Fudge to get X11::Keysyms working
 %keysymtocode = %main::keysymtocode;
@@ -158,26 +160,24 @@ sub logmsg($@) {
 
 # set some application defaults
 sub load_config_defaults() {
-  $config{terminal}           = "xterm";
-  $config{terminal_args}      = "";
-  $config{terminal_title_opt} = "-T";
-  $config{terminal_allow_send_events} =
-    "-xrm 'XTerm.VT100.allowSendEvents:true'";
-  $config{terminal_font}           = "6x13";
-  $config{terminal_size}           = "80x24";
-  $config{use_hotkeys}             = "yes";
-  $config{key_quit}                = "Control-q";
-  $config{key_addhost}             = "Control-plus";
-  $config{key_clientname}          = "Alt-n";
-  $config{key_retilehosts}         = "Alt-r";
-  $config{key_paste}               = "Control-v";
-  $config{mouse_paste}             = "Button-2";
-  $config{auto_quit}               = "yes";
-  $config{window_tiling}           = "yes";
-  $config{window_tiling_direction} = "right";
-  $config{console_position}        = "";
-
-  $config{ignore_host_errors} = "no";
+  $config{terminal}                   = "xterm";
+  $config{terminal_args}              = "";
+  $config{terminal_title_opt}         = "-T";
+  $config{terminal_allow_send_events} = "-xrm '*.VT100.allowSendEvents:true'";
+  $config{terminal_font}              = "6x13";
+  $config{terminal_size}              = "80x24";
+  $config{use_hotkeys}                = "yes";
+  $config{key_quit}                   = "Control-q";
+  $config{key_addhost}                = "Control-plus";
+  $config{key_clientname}             = "Alt-n";
+  $config{key_history}                = "Alt-h";
+  $config{key_retilehosts}            = "Alt-r";
+  $config{key_paste}                  = "Control-v";
+  $config{mouse_paste}                = "Button-2";
+  $config{auto_quit}                  = "yes";
+  $config{window_tiling}              = "yes";
+  $config{window_tiling_direction}    = "right";
+  $config{console_position}           = "";
 
   $config{screen_reserve_top}    = 0;
   $config{screen_reserve_bottom} = 40;
@@ -194,6 +194,7 @@ sub load_config_defaults() {
 
   ( $config{comms} = basename($0) ) =~ s/^.//;
   $config{comms} =~ s/.pl$//;    # for when testing directly out of cvs
+  $config{method} = $config{comms};
 
   $config{title} = "C" . uc( $config{comms} );
 
@@ -212,9 +213,9 @@ sub load_config_defaults() {
 
   $config{unmap_on_redraw} = "no";    # Debian #329440
 
-  $config{show_history} = 0;
-  $config{history_width} = 40,
-  $config{history_height} = 10,
+  $config{show_history}   = 0;
+  $config{history_width}  = 40;
+  $config{history_height} = 10;
 }
 
 # load in config file settings
@@ -225,9 +226,9 @@ sub parse_config_file($) {
 
   open( CFG, $config_file ) or die("Couldnt open $config_file: $!");
   while (<CFG>) {
-    next if ( /^\s*$/ || /^#/ );      # ignore blank lines & commented lines
-    s/#.*//;                          # remove comments from remaining lines
-    s/\s*$//;                         # remove trailing whitespace
+    next if ( /^\s*$/ || /^#/ );    # ignore blank lines & commented lines
+    s/#.*//;                        # remove comments from remaining lines
+    s/\s*$//;                       # remove trailing whitespace
     chomp();
 
     #my ($key, $value) = split(/[ 	]*=[ 	]*/);
@@ -237,6 +238,9 @@ sub parse_config_file($) {
     logmsg( 3, "$key=$value" );
   }
   close(CFG);
+
+  # tidy up entries, just in case
+  $config{terminal_font} =~ s/['"]//g;
 }
 
 sub find_binary($) {
@@ -330,8 +334,6 @@ sub check_config() {
       "-xrm '$1.VT100.allowSendEvents:true'";
   }
 
-  $config{ignore_host_errors} = "yes" if ( $options{i} );
-
   $config{internal_previous_state} = "";    # set to default
   get_font_size();
 
@@ -341,7 +343,7 @@ sub check_config() {
 }
 
 sub load_configfile() {
-  parse_config_file('/etc/csshrc');
+  parse_config_file( $sysconfigdir . '/csshrc' );
   parse_config_file( $ENV{HOME} . '/.csshrc' );
   check_config();
 }
@@ -355,21 +357,45 @@ sub dump_config {
   print("# Configuration dump produced by 'cssh -u'\n");
 
   foreach ( sort( keys(%config) ) ) {
-    next if ( $_ =~ /^internal/ && $debug == 0 );  # do not output internal vars
+    next
+      if ( $_ =~ /^internal/ && $debug == 0 );    # do not output internal vars
     print "$_=$config{$_}\n";
   }
   exit_prog if ( !$noexit );
 }
 
+sub check_ssh_hostnames {
+  return unless ( $config{method} eq "ssh" );
+
+  my $ssh_config = "$ENV{HOME}/.ssh/config";
+
+  if ( -r $ssh_config && open( SSHCFG, "<", $ssh_config ) ) {
+    while (<SSHCFG>) {
+      next unless (m/^\s*host\s+([\w\.-]+)/);
+      $ssh_hostnames{$1} = 1;
+    }
+    close(SSHCFG);
+  }
+
+  if ( $debug > 1 ) {
+    if (%ssh_hostnames) {
+      logmsg( 2, "Parsed these ssh config hosts:" );
+      logmsg( 2, "- $_" ) foreach ( sort( keys(%ssh_hostnames) ) );
+    }
+    else {
+      logmsg( 2, "No hostnames parsed from user ssh config file" );
+    }
+  }
+}
+
 sub evaluate_commands {
   my ( $return, $user, $port, $host );
 
   # break apart the given host string to check for user or port configs
   print "{e}=$options{e}\n";
-  $user = $1 if ( $options{e} =~ m/^(\w+)@\w+/ );
-  $port = $1 if ( $options{e} =~ s/:(\d+)// );
+  $user = $1 if ( $options{e} =~ s/^(.*)@// );
+  $port = $1 if ( $options{e} =~ s/:(\w+)$// );
   $host = $options{e};
-  $host =~ s/^(?:\w+@)(\w+)(?::\d+)$/$1/;
 
   $user = $user ? "-l $user" : "";
   $port = $port ? "-p $port" : "" unless ( $config{comms} eq "telnet" );
@@ -419,33 +445,42 @@ sub load_keyboard_map() {
   logmsg( 1, "Loading keymaps and keycodes" );
 
   foreach ( 0 .. $#keyboard ) {
-    if ( defined( $keycodetosym{ $keyboard[$_][0] } ) ) {
-      $keyboardmap{ $keycodetosym{ $keyboard[$_][0] } } = 'n' . ( $_ + $min );
+    if ( defined $keyboard[$_][0] ) {
+      if ( defined( $keycodetosym{ $keyboard[$_][0] } ) ) {
+        $keyboardmap{ $keycodetosym{ $keyboard[$_][0] } } = 'n' . ( $_ + $min );
+      }
+      else {
+        logmsg( 2, "Unknown keycode ", $keyboard[$_][0] )
+          if ( $keyboard[$_][0] != 0 );
+      }
     }
-    else {
-      logmsg( 2, "Unknown keycode ", $keyboard[$_][0] )
-        if ( $keyboard[$_][0] != 0 );
+    if ( defined $keyboard[$_][1] ) {
+      if ( defined( $keycodetosym{ $keyboard[$_][1] } ) ) {
+        $keyboardmap{ $keycodetosym{ $keyboard[$_][1] } } = 's' . ( $_ + $min );
+      }
+      else {
+        logmsg( 2, "Unknown keycode ", $keyboard[$_][1] )
+          if ( $keyboard[$_][1] != 0 );
+      }
     }
-    if ( defined( $keycodetosym{ $keyboard[$_][1] } ) ) {
-      $keyboardmap{ $keycodetosym{ $keyboard[$_][1] } } = 's' . ( $_ + $min );
+    if ( defined $keyboard[$_][2] ) {
+      if ( defined( $keycodetosym{ $keyboard[$_][2] } ) ) {
+        $keyboardmap{ $keycodetosym{ $keyboard[$_][2] } } = 'a' . ( $_ + $min );
+      }
+      else {
+        logmsg( 2, "Unknown keycode ", $keyboard[$_][2] )
+          if ( $keyboard[$_][2] != 0 );
+      }
     }
-    else {
-      logmsg( 2, "Unknown keycode ", $keyboard[$_][1] )
-        if ( $keyboard[$_][1] != 0 );
-    }
-    if ( defined( $keycodetosym{ $keyboard[$_][2] } ) ) {
-      $keyboardmap{ $keycodetosym{ $keyboard[$_][2] } } = 'a' . ( $_ + $min );
-    }
-    else {
-      logmsg( 2, "Unknown keycode ", $keyboard[$_][2] )
-        if ( $keyboard[$_][2] != 0 );
-    }
-    if ( defined( $keycodetosym{ $keyboard[$_][3] } ) ) {
-      $keyboardmap{ $keycodetosym{ $keyboard[$_][3] } } = 'sa' . ( $_ + $min );
-    }
-    else {
-      logmsg( 2, "Unknown keycode ", $keyboard[$_][3] )
-        if ( $keyboard[$_][3] != 0 );
+    if ( defined $keyboard[$_][3] ) {
+      if ( defined( $keycodetosym{ $keyboard[$_][3] } ) ) {
+        $keyboardmap{ $keycodetosym{ $keyboard[$_][3] } } =
+          'sa' . ( $_ + $min );
+      }
+      else {
+        logmsg( 2, "Unknown keycode ", $keyboard[$_][3] )
+          if ( $keyboard[$_][3] != 0 );
+      }
     }
 
     # dont know these two key combs yet...
@@ -600,39 +635,51 @@ sub change_main_window_title() {
   $windows{main_window}->title( $config{title} . " [$number]" );
 }
 
+sub show_history() {
+  if ( $config{show_history} ) {
+    $windows{history}->packForget();
+    $config{show_history} = 0;
+  }
+  else {
+    $windows{history}->pack(
+      -fill   => "x",
+      -expand => 1,
+    );
+    $config{show_history} = 1;
+  }
+}
+
 sub update_display_text($) {
   my $char = shift;
 
-warn("config{show_history}=$config{show_history}");
+  return if ( !$config{show_history} );
 
-  return if(!$config{show_history});
+  logmsg( 2, "Dropping :$char: into display" );
 
-  logmsg( 2, "Dropping :$char: into display");
-
-  SWITCH: {
-    foreach ( $char ) {
+SWITCH: {
+    foreach ($char) {
       /^Return$/ && do {
-        $windows{history}->insert('end', "\n");
+        $windows{history}->insert( 'end', "\n" );
         last SWITCH;
       };
-  
+
       /^BackSpace$/ && do {
         $windows{history}->delete('end - 2 chars');
         last SWITCH;
       };
-  
+
       /^(:?Shift|Control|Alt)_(:?R|L)$/ && do {
         last SWITCH;
       };
 
-      length( $char ) > 1 && do {
-        $windows{history}->insert('end', chr( $keysymtocode{$char} ) ) 
-         if ($keysymtocode{$char} );
+      length($char) > 1 && do {
+        $windows{history}->insert( 'end', chr( $keysymtocode{$char} ) )
+          if ( $keysymtocode{$char} );
         last SWITCH;
       };
 
       do {
-        $windows{history}->insert('end',$char);
+        $windows{history}->insert( 'end', $char );
         last SWITCH;
       };
     }
@@ -744,8 +791,6 @@ sub setup_helper_script() {
 		{
 			\$svr =~ s/==\$//;
 			warn("\\nWARNING: failed to resolve IP address for \$svr.\\n\\n"
-				. "Either 'ignore_host_errors' or -i is set.  "
-				. "This connection may hang\\n\\n"
 			);
 			sleep 5;
 		}
@@ -776,6 +821,23 @@ sub setup_helper_script() {
   logmsg( 2, "Helper script done" );
 }
 
+sub check_host($) {
+  my $host = shift;
+  if ( $config{method} eq "ssh" ) {
+    logmsg( 1, "Attempting name resolution via user ssh config file" );
+    if ( $ssh_hostnames{$host} ) {
+      return 1;
+    }
+    else {
+      logmsg( 1, "Failed to check host (falling back to gethost): $!" );
+      return gethost($host);
+    }
+  }
+  else {
+    return gethost($host);
+  }
+}
+
 sub open_client_windows(@) {
   foreach (@_) {
     next unless ($_);
@@ -786,13 +848,11 @@ sub open_client_windows(@) {
     my $port_nb;
 
     # split off any provided hostname and port
-    if ( $_ =~ /([\w:]+)@/ ) {
+    if ( $_ =~ s/^(.*)@// ) {
       $username = $1;
-      $_ =~ s/.*@//;
     }
-    if ( $_ =~ /:(\w+)/ ) {
+    if ( $_ =~ s/:(\w+)$// ) {
       $port_nb = $1;
-      $_ =~ s/:.*//;
     }
 
     my $count  = 1;
@@ -803,14 +863,16 @@ sub open_client_windows(@) {
     }
 
     # see if we can find the hostname - if not, drop it
-    my $gethost = gethost("$_");
+    my $gethost = check_host($_);
     if ( !$gethost ) {
-      my $text =
-"WARNING: unknown host $_ (see -i switch, or ignore_host_errors in .csshrc)";
-      $text .= " - ignoring" unless ( $config{ignore_host_errors} =~ /yes/i );
-      $text .= "\n";
-      warn($text);
-      next unless ( $config{ignore_host_errors} =~ /yes/i );
+      my $text = "WARNING: '$_' unknown";
+
+      if (%ssh_hostnames) {
+        $text .= " (unable to resolve and not in user ssh config file)";
+      }
+
+      warn( $text, $/ );
+      next;
     }
 
     $servers{$server}{realname} = $_;
@@ -822,7 +884,8 @@ sub open_client_windows(@) {
     $servers{$server}{pipenm} = tmpnam();
 
     logmsg( 2, "Set temp name to: $servers{$server}{pipenm}" );
-    mkfifo( $servers{$server}{pipenm}, 0600 ) or die("Cannot create pipe: $!");
+    mkfifo( $servers{$server}{pipenm}, 0600 )
+      or die("Cannot create pipe: $!");
 
     # NOTE: the pid is re-fetched from the xterm window (via helper_script)
     # later as it changes and we need an accurate PID as it is widely used
@@ -1367,18 +1430,21 @@ sub create_windows() {
     -expand => 1,
     );
 
+  $windows{history} = $windows{main_window}->Scrolled(
+    "ROText",
+    -insertborderwidth => 4,
+    -width             => $config{history_width},
+    -height            => $config{history_height},
+    -state             => 'normal',
+    -takefocus         => 0,
+  );
+  $windows{history}->bindtags(undef);
+
   if ( $config{show_history} ) {
-    $windows{history} = $windows{main_window}->Scrolled("ROText",
-      -insertborderwidth => 4,
-      -width             => $config{history_width},
-      -height            => $config{history_height},
-      -state             => 'normal',
-      -takefocus         => 0,
-    )->pack(
+    $windows{history}->pack(
       -fill   => "x",
       -expand => 1,
     );
-    $windows{history}->bindtags(undef);
   }
 
   $windows{main_window}->bind( '<Destroy>' => \&exit_prog );
@@ -1418,7 +1484,8 @@ sub create_windows() {
 
       # now sent it on
       foreach my $svr ( keys(%servers) ) {
-        send_text( $svr, $paste_text ) if ( $servers{$svr}{active} == 1 );
+        send_text( $svr, $paste_text )
+          if ( $servers{$svr}{active} == 1 );
       }
     }
   );
@@ -1557,7 +1624,8 @@ sub key_event {
   logmsg( 3, "codetosym=$keycodetosym{$keysymdec}" )
     if ( $keycodetosym{$keysymdec} );
   logmsg( 3, "symtocode=$keysymtocode{$keysym}" );
-  logmsg( 3, "keyboard =$keyboardmap{ $keysym }" ) if ( $keyboardmap{$keysym} );
+  logmsg( 3, "keyboard =$keyboardmap{ $keysym }" )
+    if ( $keyboardmap{$keysym} );
 
   #warn("debug stop point here");
   if ( $config{use_hotkeys} eq "yes" ) {
@@ -1579,6 +1647,7 @@ sub key_event {
           send_clientname()     if ( $hotkey eq "key_clientname" );
           add_host_by_name()    if ( $hotkey eq "key_addhost" );
           retile_hosts("force") if ( $hotkey eq "key_retilehosts" );
+          show_history()        if ( $hotkey eq "key_history" );
           exit_prog()           if ( $hotkey eq "key_quit" );
         }
         return;
@@ -1589,7 +1658,7 @@ sub key_event {
   # look for a <Control>-d and no hosts, so quit
   exit_prog() if ( $state =~ /Control/ && $keysym eq "d" and !%servers );
 
-  update_display_text( $keycodetosym{$keysymdec} ) 
+  update_display_text( $keycodetosym{$keysymdec} )
     if ( $event eq "KeyPress" && $keycodetosym{$keysymdec} );
 
   # for all servers
@@ -1628,6 +1697,12 @@ sub create_menubar() {
   $menus{file} = $menus{bar}->cascade(
     -label     => 'File',
     -menuitems => [
+      [
+        "command",
+        "Show History",
+        -command     => \&show_history,
+        -accelerator => $config{key_history},
+      ],
       [
         "command",
         "Exit",
@@ -1739,6 +1814,8 @@ load_config_defaults();
 load_configfile();
 dump_config() if ( $options{u} );
 
+check_ssh_hostnames();
+
 evaluate_commands() if ( $options{e} );
 
 load_keyboard_map();
@@ -1937,7 +2014,7 @@ Use supplied file as additional cluster file (see also L<"FILES">)
 =item -l $LOGNAME
 
 Specify the default username to use for connections (if different from the
-currently logged in user).  NOTE: will be overridden by <user>@<host>
+currently logged in user).  B<NOTE:> will be overridden by <user>@<host>
 
 =item -T "CSSH"
 
@@ -1949,7 +2026,7 @@ Specify the initial part of the title used in the console and client windows
 
 Specify arguments to be passed to ssh or rsh when making the connection.  
 
-NOTE: any "generic" change to the method (i.e. specifying the ssh port to use)
+B<NOTE:> any "generic" change to the method (i.e. specifying the ssh port to use)
 should be done in the medium's own config file (see L<ssh_config> and 
 F<$HOME/.ssh/config>).
 
@@ -1964,9 +2041,8 @@ Specify arguments to be passed to terminals being used
 
 =item -i
 
-Ignore errors from unresolvable host names (i.e. because the name is an alias
-within an ssh config file or similar) (see also "ignore_host_errors" in 
-L<"FILES">)
+THIS OPTION IS DEPRECATED.  It has been left in so current systems continue 
+to function as expected.
 
 =item -e [user@]<hostname>[:port]
 
@@ -2054,6 +2130,15 @@ This file contains configuration overrides - the defaults are as marked.
 Default options are overwritten first by the global file, and then by the
 user file.
 
+B<NOTE:> values for entries do not need to be quoted unless it is required 
+for passing arguments, i.e.
+
+  terminal_allow_send_events="-xrm '*.VT100.allowSendEvents:true'"
+
+should be written as 
+
+  terminal_allow_send_events=-xrm '*.VT100.allowSendEvents:true'
+
 =over
 
 =item always_tile = yes
@@ -2096,11 +2181,10 @@ as a to reference the users home directory, i.e.
 
  extra_cluster_file = ~/clusters, $HOME/clus
 
-=item ignore_host_errors = "no"
+=item ignore_host_errors
 
-If set to "yes", ignore errors from host names that cannot be resolved 
-and attempt to connect anyway (i.e. because they are aliased in an ssh 
-config file) - see also "-i"
+THIS OPTION IS DEPRECATED.  It has been left in so current systems continue 
+to function as expected.
 
 =item key_addhost = Control-plus
 
@@ -2137,9 +2221,9 @@ See below notes on shortcuts.
 Sets any arguments to be used with the communication method (defaults to ssh
 arguments).  
 
-NOTE: The given defaults are based on OpenSSH, not commercial ssh software.
+B<NOTE:> The given defaults are based on OpenSSH, not commercial ssh software.
 
-NOTE: Any "generic" change to the method (i.e. specifying the ssh port to use)
+B<NOTE:> Any "generic" change to the method (i.e. specifying the ssh port to use)
 should be done in the medium's own config file (see L<ssh_config> and 
 F<$HOME/.ssh/config>).
 
@@ -2196,7 +2280,7 @@ decreased when resizing terminals for tiling, not the number of characters (80))
 
 Option used with C<terminal> to set the title of the window
 
-=item terminal_allow_send_events = -xrm 'XTerm.VT100.allowSendEvents:true'
+=item terminal_allow_send_events = -xrm '*.VT100.allowSendEvents:true'
 
 Option required by the terminal to allow XSendEvents to be received
 
@@ -2231,7 +2315,7 @@ left and then up
 
 =back
 
-NOTE: The key shortcut modifiers must be in the form "Control", "Alt", or 
+B<NOTE:> The key shortcut modifiers must be in the form "Control", "Alt", or 
 "Shift", i.e. with the first letter capitalised and the rest lower case.  Keys
 may also be disabled individually by setting to the work "null".
 
